#!/bin/bash

#设置redis安装目录
while read -p"请输入redis安装路径(按q退出):" redis_dir;do
  if [ "$redis_dir" == "" ];then
    echo "值为空,请重新输入"
  elif [ "$redis_dir" == "q" ];then
    echo "现在退出redis安装程序"
    exit 10
  else
    echo "安装目录设置完毕"
    break
  fi
done

#设置redis端口
while read -p"请输入redis安装端口:" redis_port;do
  if [ "$redis_port" == "" ];then
    echo "值为空,请重新输入"
  elif [ "$redis_dir" = "q" ];then
    echo "现在退出redis安装程序"
    exit 10
  else
#    lsof -i:$redis_port &>/dev/null
    netstat -lntp | grep -w '$redis_port' &>/dev/null
    if [ $? -eq 0 ];then
      echo "端口已被占用,请重新输入"
    else
      echo "端口设置完毕"
      break
    fi
  fi
done

#建立redis用户
useradd redis -d $redis_dir/redis

sys(){
#进行系统配置调优
grep '^vm.overcommit_memory.*$' /etc/sysctl.conf &>/dev/null
if [ $? -eq 0 ];then
  vm_value=`grep '^vm.overcommit_memory.*$' /etc/sysctl.conf | awk -F"=" '{print $2}' | sed "s/^[ \t]*//g"`
  if [ "$vm_value" != 1 ];then
    sed -i "s/^vm.overcommit_memory.*$/vm.overcommit_memory = 1/g" /etc/sysctl.conf
    sysctl -p
  fi
else
   echo "vm.overcommit_memory = 1" >> /etc/sysctl.conf
   sysctl -p
fi

egrep -w "^never$" /sys/kernel/mm/transparent_hugepage/enabled &>/dev/null
if [ $? -ne 0 ];then
  echo "never" >> /sys/kernel/mm/transparent_hugepage/enabled
fi
}
sys

redis_ins(){
#安装redis
make clean
make && make PREFIX=/$redis_dir/redis install
if [ $? -eq 0 ];then
  echo "redis安装成功"
else
  echo "redis安装失败,请检查"
fi
}
redis_ins

file_con(){
#复制配置文件及启停控制文件
cp redis.conf $redis_dir/redis/${redis_port}.conf
cp utils/redis_init_script /etc/init.d/redis_$redis_port

#启动文件
sed -i "s&^REDISPORT.*$&REDISPORT=$redis_port&g" /etc/init.d/redis_$redis_port
sed -i "s&^EXEC.*$&EXEC=$redis_dir\/redis\/bin\/redis-server&g" /etc/init.d/redis_$redis_port
sed -i "s&^CONF.*$&CONF=$redis_dir\/redis\/${redis_port}.conf&g" /etc/init.d/redis_$redis_port
sed -i "s&^CLIEXEC.*$&CLIEXEC=$redis_dir\/redis\/bin\/redis-cli&g" /etc/init.d/redis_$redis_port
sed -i "s&^PIDFILE.*$&PIDFILE=$redis_dir\/redis\/${redis_port}.pid&g" /etc/init.d/redis_$redis_port

#redis配置文件
cd $redis_dir/redis
mkdir log
mkdir backup
sed -i "s&^port.*$&port $redis_port&g" $redis_dir/redis/${redis_port}.conf
sed -i "s&^daemonize.*$&daemonize yes&g" $redis_dir/redis/${redis_port}.conf
sed -i "s&^pidfile.*$&pidfile $redis_dir\/redis\/${redis_port}.pid&g" $redis_dir/redis/${redis_port}.conf
sed -i "s&^logfile.*$&logfile $redis_dir\/redis\/log\/redis.log&g" $redis_dir/redis/${redis_port}.conf

chown -R redis:redis $redis_dir/redis
}

file_con

#redis备份
rebak(){
#rdb备份
rerdb(){
sed -i "s/^save/#save/g" $redis_dir/redis/${redis_port}.conf
#sed -i "/^#save/isave 900 1/" $redis_dir/redis/${redis_port}.conf
sed -i "/#   save \"\"/asave 900 1" $redis_dir/redis/${redis_port}.conf
sed -i "s/^stop-writes-on-bgsave-error.*$/stop-writes-on-bgsave-error no/g" $redis_dir/redis/${redis_port}.conf
sed -i "s/^rdbcompression.*$/rdbcompression yes/g" $redis_dir/redis/${redis_port}.conf
sed -i "s/^dbfilename.*$/dbfilename dump.rdb/g"  $redis_dir/redis/${redis_port}.conf
sed -i "s&^dir.*$&dir $redis_dir\/redis\/backup&g" $redis_dir/redis/${redis_port}.conf
}

reaof(){
sed -i "s/^appendonly.*$/appendonly yes/g" $redis_dir/redis/${redis_port}.conf
sed -i "s/^appendfsync/#appendfsync/g" $redis_dir/redis/${redis_port}.conf
sed -i "/^#appendfsync/iappendfsync everysec" $redis_dir/redis/${redis_port}.conf
sed -i "s&^dir.*$&dir $redis_dir\/redis\/backup&g" $redis_dir/redis/${redis_port}.conf
}


while read -p"是否设置备份[y/n]:" jubak;do
  if [ "$jubak" = "y" ];then
    cat <<-EOF
    1:rdb
    2:aof
    3:ALL
    q:退出
EOF
    while read -p"请选择开启rdb备份/aof备份:" jumeth;do
      case "$jumeth" in
      1)
      rerdb
      break
      ;;
      2)
      reaof
      break
      ;;
      3)
      rerdb
      reaof
      break
      ;;
      q)
      return 1
      ;;
      *)
      echo "value is error"
      ;;
      esac
    done
    break 
 elif [ "$jubak" = "n" ];then
    return 1
#   break
  else
    echo "value is error"
  fi
done
echo "redis备份已设置完毕"

}
rebak

echo "redis安装完毕,请切换到redis用户启动"
